@@ -4,6 +4,7 @@ API for TensorFlow usage.
 
 from typing import Any, Callable, Optional, Sequence, Tuple, TypeVar
 
+import functools
 import numpy as np  # type: ignore
 import tensorflow as tf  # type: ignore
 
@@ -16,6 +17,7 @@ from targetran._functional import (
 )
 
 from targetran._transform import (
+    _affine_transform,
     _flip_left_right,
     _flip_up_down,
     _rotate,
@@ -24,7 +26,12 @@ from targetran._transform import (
     _get_random_crop_inputs,
     _get_random_size_fractions,
     _crop,
-    _resize
+    _resize,
+    _get_flip_left_right_mats,
+    _get_flip_up_down_mats,
+    _get_rotate_mats,
+    _get_shear_mats,
+    _get_translate_mats
 )
 
 T = TypeVar("T", np.ndarray, tf.Tensor)
@@ -73,6 +80,24 @@ def seqs_to_tf_dataset(
     return ds
 
 
+def _tf_affine_transform(
+        image: tf.Tensor,
+        bboxes: tf.Tensor,
+        labels: tf.Tensor,
+        image_dest_tran_mat: tf.Tensor,
+        bboxes_tran_mat: tf.Tensor
+) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
+    return _affine_transform(
+        image, bboxes, labels,
+        _tf_convert, tf.shape, tf.reshape, tf.expand_dims, tf.squeeze,
+        _tf_pad_image, tf.range, _tf_round_to_int, tf.repeat, tf.tile,
+        tf.ones_like, tf.stack, tf.concat,
+        image_dest_tran_mat, bboxes_tran_mat, tf.matmul, tf.clip_by_value,
+        _tf_gather_image, tf.identity, tf.reduce_max, tf.reduce_min,
+        tf.logical_and, tf.boolean_mask
+    )
+
+
 def tf_flip_left_right(
         image: tf.Tensor,
         bboxes: tf.Tensor,
@@ -110,10 +135,11 @@ def tf_rotate(
         angle_deg: float
 ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
     return _rotate(
-        image, bboxes, labels, angle_deg,
-        _tf_convert, tf.shape, tf.reshape, tf.expand_dims, tf.squeeze,
-        _tf_pad_image, tf.range, _tf_round_to_int, tf.repeat, tf.tile,
-        tf.ones_like, tf.stack, tf.concat, tf.cos, tf.sin, tf.matmul,
+        image, bboxes, labels, _tf_convert(angle_deg),
+        _tf_convert, tf.cos, tf.sin, tf.shape, tf.reshape,
+        tf.expand_dims, tf.squeeze, _tf_pad_image, tf.range,
+        _tf_round_to_int, tf.repeat, tf.tile,
+        tf.ones_like, tf.stack, tf.concat, tf.matmul,
         tf.clip_by_value, _tf_gather_image, tf.identity,
         tf.reduce_max, tf.reduce_min, tf.logical_and, tf.boolean_mask
     )
@@ -126,10 +152,10 @@ def tf_shear(
         angle_deg: float
 ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
     return _shear(
-        image, bboxes, labels, angle_deg,
-        _tf_convert, tf.shape, tf.reshape, tf.expand_dims, tf.squeeze,
+        image, bboxes, labels, _tf_convert(angle_deg),
+        _tf_convert, tf.tan, tf.shape, tf.reshape, tf.expand_dims, tf.squeeze,
         _tf_pad_image, tf.range, _tf_round_to_int, tf.repeat, tf.tile,
-        tf.ones_like, tf.stack, tf.concat, tf.tan, tf.matmul, tf.clip_by_value,
+        tf.ones_like, tf.stack, tf.concat, tf.matmul, tf.clip_by_value,
         _tf_gather_image, tf.identity, tf.reduce_max, tf.reduce_min,
         tf.logical_and, tf.boolean_mask
     )
@@ -144,7 +170,7 @@ def tf_translate(
 ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
     return _translate(
         image, bboxes, labels,
-        translate_height, translate_width,
+        _tf_convert(translate_height), _tf_convert(translate_width),
         _tf_convert, tf.shape, tf.reshape, tf.expand_dims, tf.squeeze,
         _tf_pad_image, tf.range, _tf_round_to_int, tf.repeat, tf.tile,
         tf.ones_like, tf.stack, tf.concat, tf.matmul, tf.clip_by_value,
@@ -177,8 +203,8 @@ def tf_crop(
 ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
     return _crop(
         image, bboxes, labels,
-        offset_height, offset_width,
-        cropped_image_height, cropped_image_width,
+        _tf_convert(offset_height), _tf_convert(offset_width),
+        _tf_convert(cropped_image_height), _tf_convert(cropped_image_width),
         _tf_convert, tf.shape, tf.reshape, tf.concat,
         tf.logical_and, tf.squeeze, tf.clip_by_value, tf.boolean_mask
     )
@@ -210,6 +236,9 @@ class TFRandomTransform:
             else tf.random.Generator.from_non_deterministic_state()
         self._rand_fn: Callable[..., tf.Tensor] = lambda: self._rng.uniform([])
 
+    def get_mats(self, image: tf.Tensor) -> Tuple[tf.Tensor, tf.Tensor]:
+        pass
+
     def __call__(
             self,
             image: tf.Tensor,
@@ -228,6 +257,39 @@ class TFRandomTransform:
         return image, bboxes, labels
 
 
+class TFCombineAffine(TFRandomTransform):
+
+    def __init__(
+            self,
+            transforms: Sequence[TFRandomTransform],
+            probability: float = 0.7,
+            seed: Optional[int] = None
+    ) -> None:
+        self._transforms = transforms
+        super().__init__(_tf_affine_transform, probability, seed)
+
+    def _combine_mats(self, image: tf.Tensor) -> Tuple[tf.Tensor, tf.Tensor]:
+        image_dest_tran_mats, bboxes_tran_mats = tuple(zip(
+            *[tran.get_mats(image) for tran in self._transforms]
+        ))
+        image_dest_tran_mat = functools.reduce(tf.matmul, image_dest_tran_mats)
+        bboxes_tran_mat = functools.reduce(tf.matmul, bboxes_tran_mats)
+        return image_dest_tran_mat, bboxes_tran_mat
+
+    def __call__(
+            self,
+            image: tf.Tensor,
+            bboxes: tf.Tensor,
+            labels: tf.Tensor,
+            *args: Any,
+            **kwargs: Any
+    ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
+        image_dest_tran_mat, bboxes_tran_mat = self._combine_mats(image)
+        return super().__call__(
+            image, bboxes, labels, image_dest_tran_mat, bboxes_tran_mat
+        )
+
+
 class TFRandomFlipLeftRight(TFRandomTransform):
 
     def __init__(
@@ -237,6 +299,9 @@ class TFRandomFlipLeftRight(TFRandomTransform):
     ) -> None:
         super().__init__(tf_flip_left_right, probability, seed)
 
+    def get_mats(self, image: tf.Tensor) -> Tuple[tf.Tensor, tf.Tensor]:
+        return _get_flip_left_right_mats(_tf_convert)
+
     def __call__(
             self,
             image: tf.Tensor,
@@ -257,6 +322,9 @@ class TFRandomFlipUpDown(TFRandomTransform):
     ) -> None:
         super().__init__(tf_flip_up_down, probability, seed)
 
+    def get_mats(self, image: tf.Tensor) -> Tuple[tf.Tensor, tf.Tensor]:
+        return _get_flip_up_down_mats(_tf_convert)
+
     def __call__(
             self,
             image: tf.Tensor,
@@ -280,6 +348,15 @@ class TFRandomRotate(TFRandomTransform):
         assert angle_deg_range[0] < angle_deg_range[1]
         self.angle_deg_range = angle_deg_range
 
+    def _get_angle_deg(self) -> tf.Tensor:
+        return _tf_convert(self.angle_deg_range[1] - self.angle_deg_range[0]) \
+               * self._rand_fn() + _tf_convert(self.angle_deg_range[0])
+
+    def get_mats(self, image: tf.Tensor) -> Tuple[tf.Tensor, tf.Tensor]:
+        return _get_rotate_mats(
+            self._get_angle_deg(), _tf_convert, tf.cos, tf.sin
+        )
+
     def __call__(
             self,
             image: tf.Tensor,
@@ -307,6 +384,13 @@ class TFRandomShear(TFRandomTransform):
         assert -90.0 < angle_deg_range[0] < angle_deg_range[1] < 90.0
         self.angle_deg_range = angle_deg_range
 
+    def _get_angle_deg(self) -> tf.Tensor:
+        return _tf_convert(self.angle_deg_range[1] - self.angle_deg_range[0]) \
+               * self._rand_fn() + _tf_convert(self.angle_deg_range[0])
+
+    def get_mats(self, image: tf.Tensor) -> Tuple[tf.Tensor, tf.Tensor]:
+        return _get_shear_mats(self._get_angle_deg(), _tf_convert, tf.tan)
+
     def __call__(
             self,
             image: tf.Tensor,
@@ -315,11 +399,7 @@ class TFRandomShear(TFRandomTransform):
             *args: Any,
             **kwargs: Any
     ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
-        angle_deg = \
-            _tf_convert(self.angle_deg_range[1] - self.angle_deg_range[0]) \
-            * self._rand_fn() + _tf_convert(self.angle_deg_range[0])
-
-        return super().__call__(image, bboxes, labels, angle_deg)
+        return super().__call__(image, bboxes, labels, self._get_angle_deg())
 
 
 class TFRandomTranslate(TFRandomTransform):
@@ -335,6 +415,30 @@ class TFRandomTranslate(TFRandomTransform):
         self.translate_height_fraction_range = translate_height_fraction_range
         self.translate_width_fraction_range = translate_width_fraction_range
 
+    def _get_translate_height_and_width(
+            self,
+            image: tf.Tensor
+    ) -> Tuple[tf.Tensor, tf.Tensor]:
+        height_fraction, width_fraction = _get_random_size_fractions(
+            self.translate_height_fraction_range,
+            self.translate_width_fraction_range,
+            self._rand_fn, _tf_convert
+        )
+        translate_height = _tf_round_to_int(
+            _tf_convert(tf.shape(image)[0]) * height_fraction
+        )
+        translate_width = _tf_round_to_int(
+            _tf_convert(tf.shape(image)[1]) * width_fraction
+        )
+        return translate_height, translate_width
+
+    def get_mats(self, image: tf.Tensor) -> Tuple[tf.Tensor, tf.Tensor]:
+        translate_height, translate_width = \
+            self._get_translate_height_and_width(image)
+        return _get_translate_mats(
+            translate_height, translate_width, _tf_convert
+        )
+
     def __call__(
             self,
             image: tf.Tensor,
@@ -343,19 +447,8 @@ class TFRandomTranslate(TFRandomTransform):
             *args: Any,
             **kwargs: Any
     ) -> Tuple[tf.Tensor, tf.Tensor, tf.Tensor]:
-        height_fraction, width_fraction = _get_random_size_fractions(
-            self.translate_height_fraction_range,
-            self.translate_width_fraction_range,
-            self._rand_fn, _tf_convert
-        )
-
-        translate_height = _tf_round_to_int(
-            _tf_convert(tf.shape(image)[0]) * height_fraction
-        )
-        translate_width = _tf_round_to_int(
-            _tf_convert(tf.shape(image)[1]) * width_fraction
-        )
-
+        translate_height, translate_width = \
+            self._get_translate_height_and_width(image)
         return super().__call__(
             image, bboxes, labels, translate_height, translate_width
         )
